/**
 * Copyright 2007 ZeroTurnaround.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

package org.zeroturnaround.javarebel;

import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.Configuration;
import org.zeroturnaround.javarebel.Integration;
import org.zeroturnaround.javarebel.Logger;
import org.zeroturnaround.javarebel.Reloader;

/**
 * Represents a JavaRebel plugin that can be used by a third-party 
 * to configure or integrate with JavaRebel. Will be called before the
 * first class is loaded in the host classloader.
 * 
 * There are two ways to register a JavaRebel plugin. One is by creating
 * a JAR file with a custom manifest entry {@link #MANIFEST_ATTRIBUTE_PLUGIN} 
 * and putting it in the classpath. The other is by setting a system property
 * {@link #SYSPROP_PLUGIN} to the plugin class.
 * 
 * @since 1.2
 * 
 * @author Jevgeni Kabanov (ekabanov@zeroturnaround.com)
 * 
 * @see Integration
 * @see Configuration
 * @see Reloader
 * @see Logger
 */
public interface Plugin {
  /**
   * This system property should point to a class name that implements
   * this interface. It can be used by applications to enable additional
   * JavaRebel configuration.
   * <br/>
   * For example <code>-Drebel.plugin=com.mycompany.mypackage.MyJavaRebelPlugin</code>.
   */
  public static final String SYSPROP_PLUGIN = "rebel.plugin";
  
  /**
   * This system property should point to comma-separated list of plugin jars that
   * implement the {@link #checkDependencies(ClassLoader, ClassResourceSource)} method.
   * <br/>
   * For example <code>-Drebel.plugins=/path/to/plugin1.jar,/path/to/plugin2.jar</code>.
   */
  public static final String SYSPROP_PLUGINS = "rebel.plugins";
  
  /**
   * This JAR MANIFEST.MF attribute should point to a class name that implements
   * this interface. It can be used by third-party libraries to enable additional
   * JavaRebel configuration or integration and also to create standalone JavaRebel
   * plugin JARs.
   */
  public static final String MANIFEST_ATTRIBUTE_PLUGIN = "JavaRebel-Plugin";
  
  /**
   * Called before the first class in the host classloader is loaded.
   */
  void preinit();
  
  /**
   * Should return whether or not this plugin can be loaded in this classloader.
   * Usually one will check for particular resources using the 
   * {@link ClassResourceSource}. 
   * 
   * @since 2.0
   */
  boolean checkDependencies(ClassLoader cl, ClassResourceSource crs);
}
