/**
 * Copyright 2007 ZeroTurnaround.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

package org.zeroturnaround.javarebel;

import java.io.InputStream;
import java.net.URL;
import java.util.Set;

/**
 * <p>Provides hooks for integrating JavaRebel with custom servlet containers.
 * To use it acquire an instance from {@link ServletIntegrationFactory}</p>
 * 
 * @see ServletIntegrationFactory
 * @see Integration
 */
public interface ServletIntegration {
  
  /**
   * Registers a <code>ServletContext</code> with JavaRebel.
   * It is necessary to do this before any calls to the rest of the methods of <code>ServletIntegration</code>,
   * the best place is usually the constructor.
   * The custom <code>ServletContext</code> must implement the <code>RebelServletContext</code>,
   * The latter can be easily added by the corresponding <code>ClassBytecodeProcessor</code>.
   * 
   * @param sc The custom servlet context being integrated.
   */
  public void registerServletContext(RebelServletContext sc);
  
  /**
   * Unregisters a <code>ServletContext</code> from JavaRebel.
   * It is recommended to do this in case the corresponding <code>ServletContext</code> is destroyed
   * (e.g. the web application is undeployed).
   * 
   * @param sc The custom servlet context being integrated.
   */
  public void unregisterServletContext(RebelServletContext sc);
  
  /**
   * Returns <code>true</code> if the given resource is managed by JavaRebel. 
   * <p>
   * In that case resources can be resolved using the corresponding methods of the <code>ServletIntegration</code>.
   * If any of these methods returns a <code>null</code> it must be returned by the <code>ServletContext</code> as well.
   * 
   * @param sc The custom servlet context being integrated.
   * @param path path of the resource.
   * @return <code>true</code> if the given resource is managed by JavaRebel.
   */
  public boolean isResourceReplaced(RebelServletContext sc, String path);
  
  /**
   * Resolves resources managed by JavaRebel.
   * 
   * @param sc The custom servlet context being integrated.
   * @param path path of the resource.
   * @return Resolved resource InputStream or <b>null</b>.
   */
  public InputStream getResourceAsStream(RebelServletContext sc, String path);
  
  /**
   * Resolves resources managed by JavaRebel.
   * 
   * @param sc The custom servlet context being integrated.
   * @param path path of the resource.
   * @return Resolved resource URL or <b>null</b>.
   */  
  public URL getResource(RebelServletContext sc, String path);
  
  /**
   * Resolves resources managed by JavaRebel.
   * 
   * @param sc The custom servlet context being integrated.
   * @param path path of the resource.
   * @return Resolved resource real path or <b>null</b>.
   */  
  public String getRealPath(RebelServletContext sc, String path);
  
  /**
   * Returns a directory-like listing of all the paths to resources managed by JavaRebel.
   * 
   * @param sc The custom servlet context being integrated.
   * @param path partial path used to match the resources.
   * @return Set containing the directory listing, or <code>null</code> if there are no resources
   *   in the web application whose path begins with the supplied path
   */
  public Set getResourcePaths(RebelServletContext sc, String path);

}