/**
 * Copyright 2007 ZeroTurnaround.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

package org.zeroturnaround.javarebel.support;

import java.net.URL;
import org.zeroturnaround.javarebel.ClassResource;

/**
 * A resource for a class loaded from a JAR file.
 * 
 * @author Jevgeni Kabanov (ekabanov@zeroturnaround.com)
 */
public class JarClassResource implements ClassResource {
  /**
   * Manages JAR cache.
   */
  public interface JarCacheManager {
    JarCache get(URL url);
  }
  
  /**
   * Simple interface to JAR file content.
   */
  public interface JarCache {
    byte[] getEntryBytes(String entryName);
    long getEntryLastModified(String entryName);
  }
  
  private static JarCacheManager cacheManager;
  
  static {
    try {
      Class jarCacheClass = Class.forName("com.zeroturnaround.javarebel.SDKJarResourceCacheImpl"); 
      cacheManager = (JarCacheManager) jarCacheClass.newInstance();
    }
    catch (ClassNotFoundException e) {
    }
    catch (InstantiationException e) {
      e.printStackTrace();
      throw new RuntimeException(e);
    }
    catch (IllegalAccessException e) {
      e.printStackTrace();
      throw new RuntimeException(e);
    }
    
    //"Do nothing" implementation
    if (cacheManager == null) {
      cacheManager = new JarCacheManager() {
        public org.zeroturnaround.javarebel.support.JarClassResource.JarCache get(URL url) {
          return null;
        }
      };
    }
  }
 
  private URL url;
  private JarCache cache;
  private String entryName;
  
  public JarClassResource(URL url) {
    this.url = url;
    
    String[] split = url.getPath().split("!/");
    
    /*
     * Might not be full path following the !
     * Case: http://www.zeroturnaround.com/forum/topic.php?id=117
    */
    if (split.length<2) {
   	 split = url.getPath().split("!");
    }
    
    entryName = split[1];
  }
  
  private JarCache getCache() {
    if (cache == null)
      cache = cacheManager.get(url);
    
    return cache;
  }

  public byte[] getBytes() {
    return getCache().getEntryBytes(entryName);
  }

  public long lastModified() {
    return getCache().getEntryLastModified(entryName);
  }
  
  public String getPath() {
    return url.getPath();
  }
  
  public URL toURL() {
    return url;
  }
  
  public String toString() {
    return url.toString();
  }
}
